# encoding: utf-8
import requests
import dateutil
from pymusement.park import Park
from pymusement.ride import Ride
from pymusement.show import Show
from pymusement.config.usj import strings as names

class UniversalJapan(Park):
    def __init__(self):
        super(UniversalJapan, self).__init__()

    def getName(self):
        return 'Universal Studios Japan'

    def _buildPark(self):
        page = self._get_page() 

        if 'list' not in page:
            print('USJ - nothing appeared')
            return

        for list_thing in page['list']:
            time = list_thing['wait']
            if u'分' in time:
                time = int(unicode(time).replace(u'分', ''))
            elif u'休止中' in time or u'本日終了' in time:
                time = 'CLOSED'

            for thing in list_thing['rows']:
                attr = self._build_attraction(thing, time, names)

        # Showtimes
        page = self._get_showpage()

        shows_so_far = []
        for list_thing in page['list']:
            for show in list_thing['rows']:
                if show['text'] not in shows_so_far:

                    shows_so_far.append(show['text'])
                    try:
                        show_doc = self._build_show(show, names)
                    except:
                        pass


    def _get_page(self):
        r = requests.get('http://ar02.biglobe.ne.jp/app/waittime/waittime.json')
        return r.json()

    def _get_showpage(self):
        r = requests.get('http://ar02.biglobe.ne.jp/app/waittime/schedule.json')
        return r.json()

    def _build_attraction(self, attr, time, names):
        try:
            name = names[attr['text']]
        except:
            print(unicode(attr['text']))
            name = attr['text']

        attraction = Ride()
        attraction.setName(name)

        if time == 'CLOSED':
            attraction.setClosed()
        else:
            attraction.setOpen()
            attraction.setTime(time)
        
        self.addRide(attraction)

    def _build_show(self, show, names):
        try:
            name = names[show['text']]
        except:
            print(unicode(show['text']))
            name = show['text']

        attraction = Show()
        attraction.setName(name)

        showtimes = show['schedule'].replace('/', ' ').split()
        for show in showtimes:
            time_obj = dateutil.parser.parse(show)
            attraction.setTime(time_obj)

        self.addShow(attraction)




# -*- coding: UTF-8 -*-
strings = {
        u"P1AA01" : u"La Cabane des Robinson",
        u"P1AA02" : u"Indiana Jones and the Temple of Peril",
        u"P1AA04" : u"Pirates of the Caribbean",
        u"P1DA03" : u"Autopia",
        u"P1DA04" : u"Buzz Lightyear Laser Blast",
        u"P1DA06" : u"Les Mystères du Nautilus",
        u"P1DA07" : u"Orbitron",
        u"P1DA08" : u"Space Mountain: Mission 2",
        u"P1DA09" : u"Star Tours",
        u"P1DA12" : u"Captain EO",
        u"P1NA00" : u"Alice's Curious Labyrinth",
        u"P1NA01" : u"Blanche-Neige et les Sept Nains",
        u"P1NA02" : u"Le Carrousel de Lancelot",
        u"P1NA03" : u"Casey Jr. - le Petit Train du Cirque",
        u"P1NA05" : u"Dumbo the Flying Elephant",
        u"P1NA07" : u"\"it's a small world\" Celebration",
        u"P1NA08" : u"Mad Hatter's Tea Cups",
        u"P1NA09" : u"Le Pays des Contes de Fées",
        u"P1NA10" : u"Peter Pan's Flight",
        u"P1NA13" : u"Les Voyages de Pinocchio",
        u"P1RA00" : u"Big Thunder Mountain",
        u"P1RA03" : u"Phantom Manor",
        u"P1RA04" : u"River Rogue Keelboats",
        u"P2XA00" : u"Studio Tram Tour: Behind the Magic",
        u"P2XA01" : u"CinéMagique",
        u"P2XA02" : u"Cars Quatre Roues Rallye",
        u"P2XA03" : u"Crush's Coaster",
        u"P2XA05" : u"Flying Carpets Over Agrabah",
        u"P2XA06" : u"RC Racer",
        u"P2XA07" : u"Toy Soldiers Parachute Drop",
        u"P2XA08" : u"Slinky Dog Zigzag Spin",
        u"P2ZA00" : u"Armageddon : les Effets Spéciaux",
        u"P2ZA01" : u"Rock 'n' Roller Coaster starring Aerosmith",
        u"P2ZA02" : u"The Twilight Zone Tower of Terror",
        u"P2XA09" : u"Ratatouille : L'Aventure Totalement Toquée de Rémy"
}



from pymusement.parks.universal.UniversalPark import UniversalPark

class UniversalVolcano(UniversalPark):
    def __init__(self):
        super(UniversalVolcano, self).__init__()

    def getId(self):
        return 13801
    def getCity(self):
        return 'Orlando'
    def getName(self):
        return 'Universal Volcano Bay'



from pymusement.parks.universal.UniversalPark import UniversalPark

class IslandsOfAdventure(UniversalPark):
    def __init__(self):
        super(IslandsOfAdventure, self).__init__()

    def getId(self):
        return 10000

    def getCity(self):
        return 'Orlando'

    def getName(self):
        return 'Islands of Adventure'



from pymusement.parks.universal.UniversalPark import UniversalPark

class UniversalHollywood(UniversalPark):
    def __init__(self):
        super(UniversalHollywood, self).__init__()

    def getId(self):
        return 13825
    def getCity(self):
        return 'Hollywood'
    def getName(self):
        return 'Universal Studios Hollywood'



from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()


setup(name='pymusement',
      version='0.2',
      description='A python package and CLI to get wait times for rides at various theme parks',
      url='https://github.com/TheMarauder95/PyMusement',
      author='Josh Shrawder',
      author_email="joshshrawder@gmail.com",
      license='MIT',
      long_description=long_description,
      long_description_content_type="text/markdown",
      packages=find_packages(),
      include_package_data=True,
      download_url = 'https://github.com/TheMarauder95/pymusement/archive/0.1.0.tar.gz',
      install_requires=[
        'requests',
        'beautifulsoup4',
        'python-dateutil',
        'Click'
      ],
      entry_points= {
          'console_scripts' :
                'pymusement = pymusement.scripts.cli:cli'
                    }, 
      zip_safe=False)






import click
from pymusement import PARKS

@click.command()
@click.argument('name', nargs=1, type=click.Choice(PARKS.keys()))
@click.option('--type', type=click.Choice(['rides', 'shows']), prompt='Please choose rides or shows')
def cli(name, type):
    park = PARKS[name]
    if type == 'rides':
        print_rides(park.rides())
    if type == 'shows':
        print_shows(park.shows())

def print_rides(ride_array):
    longest_name = max(len(key['name']) for key in ride_array)
    one_closed = False in [key['isOpen'] for key in ride_array]
    for ride in ride_array:
        line = ''
        line += ride['name'] + ' ' * (longest_name - len(ride['name']))
        line += ' ' * 3
        if ride['isOpen'] is True:
            line += 'Open'
            # make sure the times are aligned b/c closed longer than open
            if one_closed:
                line += ' ' * 2
        else:
            line += 'Closed'

        line += ' ' * 5
        if ride['wait'] == -9:
            line += 'Virtual Line'
        else:
            line += str(ride['wait']) + ' mins'
        click.echo(line)

def print_shows(show_array):
    longest_name = max(len(key['name']) for key in show_array)
    for show in show_array:
        line = ''
        line += show['name'] + ' ' * (longest_name - len(show['name']))
        line += ' ' * 3
        line += '\n'
        if not show['showtimes']:
            line += 'No Shows \n'
        else:
            if show['endtimes'] is not None and show['endtimes'][0] is not None:
                for time, end in zip(show['showtimes'], show['endtimes']):
                    line += time + ' - ' + end + '\n'
                    
                        
            else:
                for time in show['showtimes']:
                    line += time + '\n'
        click.echo(line)
if __name__ == "__main__":
    cli()




class Attraction(dict):

    _keys = ['name']

    def __init__(self):
        super(Attraction, self).__init__()
        for key in self._keys:
            self[key] = None

    def __setitem__(self, key, value):
        if key not in self._keys:
            raise Exception("'" + key + "'" + " is not a valid key")
        dict.__setitem__(self, key, value)

    def _addKeys(self, array):
        for key in array:
            self._keys.append(key)
            self[key] = None

    def attributes(self):
        return self._keys

    def setName(self, name):
        self['name'] = name

    def getName(self):
        return self['name']



import requests
import datetime
import hmac
import base64
import re
import hashlib
import json
from pymusement.park import Park
from pymusement.ride import Ride
from pymusement.show import Show
SHARED_HEADERS = {
    'Accept'                          : 'application/json',
    'Accept-Language'                 : 'en-US',
    'X-UNIWebService-AppVersion'      : '1.2.1',
    'X-UNIWebService-Platform'        : 'Android',
    'X-UNIWebService-PlatformVersion' : '4.4.2',
    'X-UNIWebService-Device'          : 'samsung SM-N9005',
    'X-UNIWebService-ServiceVersion'  : '1',
    'User-Agent'                      : 'Dalvik/1.6.0 (Linux; U; Android 4.4.2; SM-N9005 Build/KOT49H)',
    'Connection'                      : 'keep-alive',
    'Accept-Encoding'                 : 'gzip'
  }

RIDE_URL = 'https://services.universalorlando.com/api/pointsofinterest/rides'
SHOW_URL = 'https://services.universalorlando.com/api/pointsofinterest/Shows'
HOUR_URL = 'https://services.universalorlando.com/api/venues/{0}/hours'

class UniversalPark(Park):
    def __init__(self):
        super(UniversalPark, self).__init__()

    def _buildPark(self):
        token = self._get_token()
        ride_page = self._get_request(token, RIDE_URL)
        show_page = self._get_request(token, SHOW_URL)
        hour_page = self._get_request(token, HOUR_URL.format(self.getId()))
        
        for date in hour_page:
            if datetime.date.fromisoformat(date['Date']) == datetime.date.today():
                open_time, close_time = datetime.datetime.fromisoformat(date['OpenTimeString']), datetime.datetime.fromisoformat(date['CloseTimeString'])
                if open_time < datetime.datetime.now().astimezone() < close_time:
                    self.set_open()
                else:self.set_closed()
        
        for ride in ride_page['Results']:
            if ride['VenueId'] == self.getId():
                self._make_attraction(ride)

        
        for show in show_page['Results']:
            if show['VenueId'] == self.getId():
                self._make_show(show)
        

    def _make_attraction(self, ride):
        attraction = Ride()
        attraction.setName(ride['MblDisplayName'])
        
        if ride['WaitTime'] is None:
            attraction.setClosed()
        else:
            attraction.setOpen()

        if ride['WaitTime'] == -50:
            attraction.setTime(-1)
            attraction.setStatus('Unknown')
            attraction.setClosed()
        if ride['WaitTime'] == -9:
            attraction.setOpen()
            attraction.setTime(0)
            attraction.setStatus('VLO')
        elif ride['WaitTime'] == -8:
            attraction.setClosed()
            attraction.setTime(-1)
            attraction.setStatus('Not Open Yet')
        elif ride['WaitTime'] == -7:
            attraction.setTime(0)
            attraction.setOpen()
        elif ride['WaitTime'] == -5:
            attraction.setClosed()
            attraction.setTime(-1)
            attraction.setStatus('Capacity')
        elif ride['WaitTime'] == -4:
            attraction.setClosed()
            attraction.setTime(-1)
            attraction.setStatus('Weather')
        elif ride['WaitTime'] < 0:
            attraction.setClosed()
            attraction.setTime(-1)
            attraction.setStatus('Closed')
            

        else:
            attraction.setTime(ride['WaitTime'])
        
        attraction.setVirtualLine(ride['VirtualLine'])
        
        
        
        self.addRide(attraction)

    def _make_show(self, show):
        show_obj = Show()
        show_obj.setName(show['MblDisplayName'])
        for time in show['StartTimes']:
            show_obj.addTime(time)
        if show['EndTimes']:
            for time in show['EndTimes']:
                show_obj.addEnd(time)
        self.addShow(show_obj)


    def _get_request(self, token, url):
        headers = {
            'X-UNIWebService-ApiKey' : 'AndroidMobileApp',
            'X-UNIWebService-Token' : token 
        }
        headers.update(SHARED_HEADERS)
        params = {'city':self.getCity(),'pageSize':'all'}
        r = requests.get(url, headers=headers, params=params)
        return r.json()

    def _get_token(self):
        # Thanks to lloydpick for the Key / Secret
        KEY    = 'AndroidMobileApp'
        SECRET = b'AndroidMobileAppSecretKey182014'

        date = datetime.datetime.utcnow().strftime("%a, %d %b %Y %H:%M:%S GMT")
        secret_test = "{KEY}\n{date}\n".format(KEY=KEY, date=date)
        digest = hmac.new(SECRET, secret_test.encode('ascii'), hashlib.sha256)
        signature = base64.b64encode(digest.digest()).strip()
        signature = re.sub('/=$/', "\u003d", signature.decode('ascii'))
        
        
        params = { 
                'apikey': 'AndroidMobileApp', 
                'signature': signature 
        }
        headers = {
            'Date' :date,
            'Content-Type' : 'application/json; charset=UTF-8'
        }

        headers.update(SHARED_HEADERS)

        r = requests.post('https://services.universalorlando.com/api', headers=headers, data=json.dumps(params, ensure_ascii=False))
        return r.json()['Token']
